#ifndef __MLNX460_IF_INT_H_
#define __MLNX460_IF_INT_H_

/* Internal i2c interface */

/* 
  Description: adding new virtual buses into i2c tree.
               Actually it's used to introduce one or several switches into the 
               system. Each switch has upper edge bus id to which it should be
               connected to, and LEGS_NUM number of its legs to which could be 
               connected lower edge i2c devices or other swithces.
               Could be added up to MAX_SW_INREQ_NUM number of switches within
               single request.
  Input:
               *params - pointer to input template (see description of 
                         struct mlnx_i2c_virt_bus_ioctl_params from module mlnx_if.h.
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               input template is allocated and released by user.
*/
int mlnx_add_i2c_virtual_buses(struct mlnx_i2c_virt_bus_ioctl_params *params);

/* 
  Description: removing virtual buses into i2c tree.
  Input:
               *params - pointer to input template (see description of 
                         struct mlnx_i2c_switch_ioctl_params from module mlnx_if.h.
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               input template is allocated and released by user.
*/
int mlnx_delete_i2c_virtual_buses(struct mlnx_i2c_switch_ioctl_params *params);

/* 
  Description: blocking exitsing virtual buses of i2c tree.
               After blocking any attempt to send a message through blocked
               bus will be rejected with EBUSY error code.
  Input:
               *params - pointer to input template (see description of 
                         struct mlnx_i2c_switch_ioctl_params from module mlnx_if.h.
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               input template is allocated and released by user.
*/
int mlnx_block_i2c_virtual_buses(struct mlnx_i2c_switch_ioctl_params *params);

/* 
  Description: unblocking previously blocked virtual buses of i2c tree.
  Input:
               *params - pointer to input template (see description of 
                         struct mlnx_i2c_switch_ioctl_params from module mlnx_if.h.
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               input template is allocated and released by user.
*/
int mlnx_unblock_i2c_virtual_buses(struct mlnx_i2c_switch_ioctl_params *params);

/* 
  Description: reset and unblock if have been blocked virtual buses, remove and add anew
               attached devices.
               Up to MAX_DEV_INREQ_NUM number of devices could be attached
               within single request.
  Input:
               *params - pointer to input template (see description of 
                         struct mlnx_i2c_switch_ioctl_params from module mlnx_if.h.
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               input template is allocated and released by user.
*/
int mlnx_reset_i2c_virtual_buses_and_devices(struct mlnx_i2c_switch_ioctl_params *params);

/* 
  Description: adding i2c devices to switch leg.
               Up to MAX_DEV_INREQ_NUM number of devices could be attached
               within single request.
  Input:
               *params - pointer to input template (see description of 
                         struct mlnx_i2c_switch_ioctl_params from module mlnx_if.h.
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               input template is allocated and released by user.
*/
int mlnx_add_devices_to_vbus(struct mlnx_i2c_devices_ioctl_params *params);

/* 
  Description: deleting i2c devices from switch leg.
               Up to MAX_DEV_INREQ_NUM number of devices could be de-attached
               within single request.
  Input:
               *params - pointer to input template (see description of 
                         struct mlnx_i2c_switch_ioctl_params from module mlnx_if.h.
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               input template is allocated and released by user.
*/
int mlnx_delete_devices_from_vbus(struct mlnx_i2c_switch_ioctl_params *params);

/* 
  Description: adding i2c gpio virtual buses.
               Up to MAX_I2C_GPIO_INREQ_NUM number of buses could be attached
               within single request.
  Input:
               *params - pointer to input template (see description of 
                         struct mlnx_gpio_i2c_ioctl_params from module mlnx_if.h.
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               input template is allocated and released by user.
*/
int mlnx_add_gpio_i2c_buses(struct mlnx_gpio_i2c_ioctl_params *params);

/* 
  Description: deleting i2c gpio virtual buses.
               Up to MAX_I2C_GPIO_INREQ_NUM number of devices could be de-attached
               within single request.
  Input:
               *params - pointer to input template (see description of 
                         struct mlnx_gpio_i2c_del_ioctl_params from module mlnx_if.h.
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               input template is allocated and released by user.
*/
int mlnx_del_gpio_i2c_buses(struct mlnx_gpio_i2c_del_ioctl_params *params);

/* 
  Description: sending i2c read or write request to devices.
  Input:
               *params - pointer to input template (see description of 
                         struct mlnx_rdwr_i2c_ioctl_params from module mlnx_if.h.
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               input template is allocated and released by user.
*/
int mlnx_rw_i2c_device(struct mlnx_rdwr_i2c_ioctl_params *params);

/* 
  Description: sending smbus read or write request to devices.
  Input:
               *params - pointer to input template (see description of 
                         struct mlnx_rdwr_smbus_ioctl_params from module mlnx_if.h.
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               input template is allocated and released by user.
*/
int mlnx_rw_smbus_device(struct mlnx_rdwr_smbus_ioctl_params *params);

/* 
  Description: read or write from LPC devices.
  Input:
               *params - pointer to input template (see description of 
                         struct mlnx_lpc_ioctl_params from module mlnx_if.h.
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               input template is allocated and released by user.
*/
int mlnx_rw_lpc_device(struct mlnx_lpc_ioctl_params *params);

/* 
  Description: read or write from LPC devices. Called from other kernel modules not through ioctl.
  Input:
               *params - pointer to input template (see description of 
                         struct mlnx_lpc_kernel_params from module mlnx_if.h.
  Return code:
               N/A - can't fail.
  Note:
               input template is allocated and released by user.
*/
void mlnx_rw_lpc_kernel(struct mlnx_lpc_kernel_params *params);

/* 
  Description: sending smbus read or write request to devices.
  Input:
               *params - pointer to input template (see description of 
                         struct mlnx_rdwr_smbus_ioctl_params_2 from module mlnx_if.h.
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               input template is allocated and released by user.
*/
int mlnx_rw_smbus(struct mlnx_rdwr_smbus_ioctl_params_2 *params);

/* 
  Description: interface function. Called from other kernel modules not through ioctl.
  Input:
               *params - pointer to input template (see description of 
                         struct mlnx_rdwr_smbus_ioctl_params_2 from module mlnx_if.h.
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               input template is allocated and released by user.
*/
int mlnx_rw_i2c_kernel(struct mlnx_rdwr_i2c_ioctl_params *params);

/* 
  Description: interface function. Called from other kernel modules.
               Blocks virtual bus to main i2c switch of module (Leaf / Spine).
  Input:
               virt_bus_num - virtual_bus_num of i2c device on module.
               state - state of bus; 0 - unblocked, 1 - blocked.
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
 */
int mlnx_block_unblock_i2c_virtual_bus_kernel(unsigned short virt_bus_num, unsigned char state);

/* 
  Description: test existance of virtual bus.
  Input:
               *params - pointer to input template (see description of 
                         struct mlnx_i2c_virt_bus_scan_ioctl_params from module mlnx_if.h.
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               input template is allocated and released by user.
*/
int mlnx_test_vbus_exist(struct mlnx_i2c_virt_bus_scan_ioctl_params *params);

/* 
  Description: test existance of virtual bus.
  Input:
               *params - pointer to input template (see description of 
                         struct mlnx_i2c_one_device_ioctl_params from module mlnx_if.h.
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               input template is allocated and released by user.
*/
int mlnx_test_device_exist(struct mlnx_i2c_one_device_ioctl_params *params);

/* 
  Description: scanning i2c devices behind i2c switch (directly attached only).
  Input:
               *params - pointer to input template (see description of 
                         struct mlnx_i2c_virt_bus_scan_ioctl_params from module mlnx_if.h.
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               input template is allocated and released by user.
*/
int mlnx_scan_switch_devices(struct mlnx_i2c_virt_bus_scan_ioctl_params *params);

/* 
  Description: adding single i2c device to virtual bus.
  Input:
               *params - pointer to input template (see description of 
                         struct mlnx_i2c_one_device_ioctl_params from module mlnx_if.h.
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               input template is allocated and released by user.
*/
int mlnx_add_one_device_to_vbus(struct mlnx_i2c_one_device_ioctl_params *params);

/* 
  Description: deleting single i2c device from virtual bus.
  Input:
               *params - pointer to input template (see description of 
                         struct mlnx_i2c_one_device_ioctl_params from module mlnx_if.h.
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               input template is allocated and released by user.
*/
int mlnx_delete_one_device_from_vbus(struct mlnx_i2c_one_device_ioctl_params *params);

/* 
  Description: subscribe action function into CPLD interrupt handler .
  Input:
               *params - pointer to input template (see description of 
                         struct mlnx_cpld_subscribe_ioctl_params from module mlnx_if.h.
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               input template is allocated and released by user.
*/
int mlnx_cpld_irq_subscribe(struct mlnx_cpld_subscribe_ioctl_params *params);

/* 
  Description: unsubscribe action function from CPLD interrupt handler .
  Input:
               *params - pointer to input template (see description of 
                         struct mlnx_cpld_subscribe_ioctl_params from module mlnx_if.h.
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               input template is allocated and released by user.
*/
int mlnx_cpld_irq_unsubscribe(struct mlnx_cpld_subscribe_ioctl_params *params);

/* 
  Description: set CPLD interrupt registers layout .
  Input:
               *params - pointer to input template (see description of 
                         struct mlnx_cpld_set_ioctl_params from module mlnx_if.h.
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               input template is allocated and released by user.
*/
int mlnx_cpld_irq_set(struct mlnx_cpld_set_ioctl_params *params);

/* 
  Description: get CPLD interrupt registers content .
  Input:
               *params - pointer to input template (see description of 
                         struct mlnx_cpld_get_data_ioctl_params from module mlnx_if.h.
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               input template is allocated and released by user.
*/
int mlnx_cpld_irq_get_data(struct mlnx_cpld_get_data_ioctl_params *params);

/*
  Description: sending i2c read or write request to VPD devices using time limit.
  Input:
               *params - pointer to input template (see description of
                         struct mlnx_rdwr_vpd_i2c_ioctl_params from module mlnx_if.h.
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note:
               input template is allocated and released by user.
*/
int mlnx_vpd_rw_limited(struct mlnx_rdwr_vpd_i2c_ioctl_params *params);

/* 
  Description: init routine.
*/
int mlnx_i2c_init(void);

/* 
  Description: exit routine.
*/
void mlnx_i2c_exit(void);


#endif /* __MLNX460_IF_INT_H_ */
